def __version__() -> str: ...
def evaluate(card1: str, card2: str, card3: str, card4: str, card5: str) -> int:
    """This function evaluates a poker hand of five cards.

    Cards are encoded as two characters:
    - one for symbol among "23456789TJQKA"
    - one for color among "CDHS"
    """

def evaluate_7(card1: str, card2: str, card3: str, card4: str, card5: str, card6: str, card7: str) -> int:
    """This function evaluates a poker hand of seven cards.

    Cards are encoded as two characters:
    - one for symbol among "23456789TJQKA"
    - one for color among "CDHS"
    """



import polars_cactus as plc


def test_polars_cactus_evaluate():
    assert plc.evaluate("AC", "4C", "5C", "TC", "AH") == 3484


def test_polars_cactus_evaluate_7():
    assert plc.evaluate_7("AC", "4C", "5C", "TC", "AH", "9H", "7S") == 3463



from __future__ import annotations

from pathlib import Path
from typing import Iterable, Protocol, cast

import polars as pl
from polars._typing import PolarsDataType
from polars.plugins import register_plugin_function

from polars_cactus._core import __version__, evaluate, evaluate_7


@pl.api.register_expr_namespace("cactus")
class CactusEvaluator:
    def __init__(self, expr: pl.Expr):
        self._expr = expr

    def translate_card_to_repr(self) -> pl.Expr:
        """Takes a string as input and returns a u32 that represents the card."""
        return register_plugin_function(
            plugin_path=Path(__file__).parent,
            function_name="translate_card_to_repr",
            args=self._expr,
            is_elementwise=True,
        )

    def translate_repr_to_card(self) -> pl.Expr:
        """Takes a u32 as input and returns a string that represents the card."""
        return register_plugin_function(
            plugin_path=Path(__file__).parent,
            function_name="translate_repr_to_card",
            args=self._expr,
            is_elementwise=True,
        )

    def evaluate_5cards_repr(self) -> pl.Expr:
        """Takes a list of u32 as input and returns a u32 that represents the hand."""
        return register_plugin_function(
            plugin_path=Path(__file__).parent,
            function_name="evaluate_5cards_repr",
            args=self._expr,
            is_elementwise=True,
        )

    def evaluate_5cards(self) -> pl.Expr:
        """Takes a list of str as input and returns a u32 that represents the hand."""
        return register_plugin_function(
            plugin_path=Path(__file__).parent,
            function_name="evaluate_5cards",
            args=self._expr,
            is_elementwise=True,
        )


class CExpr(pl.Expr):
    @property
    def cactus(self) -> CactusEvaluator:
        return CactusEvaluator(self)


class CactusColumn(Protocol):
    def __call__(
        self,
        name: str | PolarsDataType | Iterable[str] | Iterable[PolarsDataType],
        *more_names: str | PolarsDataType,
    ) -> CExpr: ...

    def __getattr__(self, name: str) -> pl.Expr: ...

    @property
    def translate_card_to_repr(self) -> CactusColumn: ...

    @property
    def translate_repr_to_card(self) -> CactusColumn: ...

    @property
    def evaluate_5cards_repr(self) -> CactusColumn: ...

    @property
    def evaluate_5cards(self) -> CactusColumn: ...


col = cast(CactusColumn, pl.col)


__all__ = ["col", "__version__", "evaluate", "evaluate_7"]



import polars as pl
from polars.testing import assert_frame_equal

import polars_cactus as plc


def test_translate_card_to_repr():
    df = pl.DataFrame({"card": ["TH", "Z", "TT"]})
    result = df.select(plc.col("card").cactus.translate_card_to_repr())
    expected = pl.Series([16787479, None, None], dtype=pl.UInt32)
    assert_frame_equal(result, pl.DataFrame({"card": expected}))


def test_translate_repr_to_card():
    df = pl.DataFrame({"repr": pl.Series([16787479, 0, 1], dtype=pl.UInt32)})
    result = df.select(plc.col("repr").cactus.translate_repr_to_card())
    expected = pl.Series(["TH", None, None])
    assert_frame_equal(result, pl.DataFrame({"repr": expected}))


def test_evaluate_5cards_repr():
    df = pl.DataFrame(
        {
            "cards": pl.Series(
                [
                    [8398611, 4204049, 2106637, 1057803, 16787479],
                    [268471337, 295429, 557831, 16812055, 268446761],
                ],
                dtype=pl.List(pl.UInt32),
            )
        }
    )
    result = df.select(plc.col("cards").cactus.evaluate_5cards_repr())
    expected = pl.Series([5, 3484], dtype=pl.UInt32)
    assert_frame_equal(result, pl.DataFrame({"cards": expected}))


def test_evaluate_5cards():
    df = pl.DataFrame(
        {
            "cards": pl.Series(
                [
                    ["9H", "8H", "7H", "6H", "TH"],
                    ["AC", "4C", "5C", "TC", "AH"],
                ]
            )
        }
    )
    result = df.select(plc.col("cards").cactus.evaluate_5cards())
    expected = pl.Series([5, 3484], dtype=pl.UInt32)
    assert_frame_equal(result, pl.DataFrame({"cards": expected}))
